@@ -5,7 +5,7 @@ puremagic is a pure python module that will identify a file based off it's
 magic numbers. It is designed to be minimalistic and inherently cross platform
 compatible, with no imports when used as a module.
 
-© 2013-2020 Chris Griffith - License: MIT (see LICENSE)
+© 2013-2022 Chris Griffith - License: MIT (see LICENSE)
 
 Acknowledgements
 Gary C. Kessler
@@ -17,9 +17,10 @@ import json
 import binascii
 from itertools import chain
 from collections import namedtuple
+from typing import Union, Tuple, List
 
 __author__ = "Chris Griffith"
-__version__ = "1.10"
+__version__ = "1.13"
 __all__ = [
     "magic_file",
     "magic_string",
@@ -35,23 +36,37 @@ __all__ = [
 
 here = os.path.abspath(os.path.dirname(__file__))
 
-MAGIC_INFO_TYPES = (
-    "byte_match",
-    "offset",
-    "extension",
-    "mime_type",
-    "name",
+PureMagic = namedtuple(
+    "PureMagic",
+    (
+        "byte_match",
+        "offset",
+        "extension",
+        "mime_type",
+        "name",
+    ),
+)
+PureMagicWithConfidence = namedtuple(
+    "PureMagicWithConfidence",
+    (
+        "byte_match",
+        "offset",
+        "extension",
+        "mime_type",
+        "name",
+        "confidence",
+    ),
 )
-PureMagic = namedtuple("PureMagic", MAGIC_INFO_TYPES)  # type: ignore
-PureMagicWithConfidence = namedtuple("PureMagicWithConfidence", (MAGIC_INFO_TYPES + ("confidence",)))  # type: ignore
 
 
 class PureError(LookupError):
     """Do not have that type of file in our databanks"""
 
 
-def _magic_data(filename=os.path.join(here, "magic_data.json")):
-    """ Read the magic file"""
+def _magic_data(
+    filename: Union[os.PathLike, str] = os.path.join(here, "magic_data.json"),
+) -> Tuple[List[PureMagic], List[PureMagic]]:
+    """Read the magic file"""
     with open(filename) as f:
         data = json.load(f)
     headers = sorted((_create_puremagic(x) for x in data["headers"]), key=lambda x: x.byte_match)
@@ -59,7 +74,7 @@ def _magic_data(filename=os.path.join(here, "magic_data.json")):
     return headers, footers
 
 
-def _create_puremagic(x):
+def _create_puremagic(x: List) -> PureMagic:
     return PureMagic(
         byte_match=binascii.unhexlify(x[0].encode("ascii")), offset=x[1], extension=x[2], mime_type=x[3], name=x[4]
     )
@@ -68,15 +83,15 @@ def _create_puremagic(x):
 magic_header_array, magic_footer_array = _magic_data()
 
 
-def _max_lengths():
-    """ The length of the largest magic string + its offset"""
+def _max_lengths() -> Tuple[int, int]:
+    """The length of the largest magic string + its offset"""
     max_header_length = max([len(x.byte_match) + x.offset for x in magic_header_array])
     max_footer_length = max([len(x.byte_match) + abs(x.offset) for x in magic_footer_array])
     return max_header_length, max_footer_length
 
 
-def _confidence(matches, ext=None):
-    """ Rough confidence based on string length and file extension"""
+def _confidence(matches, ext=None) -> List[PureMagicWithConfidence]:
+    """Rough confidence based on string length and file extension"""
     results = []
     for match in matches:
         con = 0.8 if len(match.byte_match) >= 9 else float("0.{0}".format(len(match.byte_match)))
@@ -86,8 +101,8 @@ def _confidence(matches, ext=None):
     return sorted(results, key=lambda x: x.confidence, reverse=True)
 
 
-def _identify_all(header, footer, ext=None):
-    """ Attempt to identify 'data' by its magic numbers"""
+def _identify_all(header: bytes, footer: bytes, ext=None) -> List[PureMagicWithConfidence]:
+    """Attempt to identify 'data' by its magic numbers"""
 
     # Capture the length of the data
     # That way we do not try to identify bytes that don't exist
@@ -110,8 +125,8 @@ def _identify_all(header, footer, ext=None):
     return _confidence(matches, ext)
 
 
-def _magic(header, footer, mime, ext=None):
-    """ Discover what type of file it is based on the incoming string """
+def _magic(header: bytes, footer: bytes, mime: bool, ext=None) -> str:
+    """Discover what type of file it is based on the incoming string"""
     if not header:
         raise ValueError("Input was empty")
     info = _identify_all(header, footer, ext)[0]
@@ -120,8 +135,8 @@ def _magic(header, footer, mime, ext=None):
     return info.extension if not isinstance(info.extension, list) else info[0].extension
 
 
-def _file_details(filename):
-    """ Grab the start and end of the file"""
+def _file_details(filename: Union[os.PathLike, str]) -> Tuple[bytes, bytes]:
+    """Grab the start and end of the file"""
     max_head, max_foot = _max_lengths()
     with open(filename, "rb") as fin:
         head = fin.read(max_head)
@@ -134,13 +149,13 @@ def _file_details(filename):
 
 
 def _string_details(string):
-    """ Grab the start and end of the string"""
+    """Grab the start and end of the string"""
     max_head, max_foot = _max_lengths()
     return string[:max_head], string[-max_foot:]
 
 
 def _stream_details(stream):
-    """ Grab the start and end of the stream"""
+    """Grab the start and end of the stream"""
     max_head, max_foot = _max_lengths()
     head = stream.read(max_head)
     stream.seek(-max_foot, os.SEEK_END)
@@ -148,14 +163,14 @@ def _stream_details(stream):
     return head, foot
 
 
-def ext_from_filename(filename):
-    """ Scan a filename for it's extension.
+def ext_from_filename(filename: Union[os.PathLike, str]) -> str:
+    """Scan a filename for its extension.
 
     :param filename: string of the filename
     :return: the extension off the end (empty string if it can't find one)
     """
     try:
-        base, ext = filename.lower().rsplit(".", 1)
+        base, ext = str(filename).lower().rsplit(".", 1)
     except ValueError:
         return ""
     ext = ".{0}".format(ext)
@@ -169,8 +184,8 @@ def ext_from_filename(filename):
     return ext
 
 
-def from_file(filename, mime=False):
-    """ Opens file, attempts to identify content based
+def from_file(filename: Union[os.PathLike, str], mime: bool = False) -> str:
+    """Opens file, attempts to identify content based
     off magic number and will return the file extension.
     If mime is True it will return the mime type instead.
 
@@ -183,8 +198,8 @@ def from_file(filename, mime=False):
     return _magic(head, foot, mime, ext_from_filename(filename))
 
 
-def from_string(string, mime=False, filename=None):
-    """ Reads in string, attempts to identify content based
+def from_string(string: Union[str, bytes], mime: bool = False, filename: Union[os.PathLike, str] = None) -> str:
+    """Reads in string, attempts to identify content based
     off magic number and will return the file extension.
     If mime is True it will return the mime type instead.
     If filename is provided it will be used in the computation.
@@ -194,13 +209,15 @@ def from_string(string, mime=False, filename=None):
     :param filename: original filename
     :return: guessed extension or mime
     """
+    if isinstance(string, str):
+        string = string.encode("utf-8")
     head, foot = _string_details(string)
     ext = ext_from_filename(filename) if filename else None
     return _magic(head, foot, mime, ext)
 
 
-def from_stream(stream, mime=False, filename=None):
-    """ Reads in stream, attempts to identify content based
+def from_stream(stream, mime: bool = False, filename: Union[os.PathLike, str] = None) -> str:
+    """Reads in stream, attempts to identify content based
     off magic number and will return the file extension.
     If mime is True it will return the mime type instead.
     If filename is provided it will be used in the computation.
@@ -215,8 +232,9 @@ def from_stream(stream, mime=False, filename=None):
     return _magic(head, foot, mime, ext)
 
 
-def magic_file(filename):
-    """ Returns tuple of (num_of_matches, array_of_matches)
+def magic_file(filename: Union[os.PathLike, str]) -> List[PureMagicWithConfidence]:
+    """
+    Returns list of (num_of_matches, array_of_matches)
     arranged highest confidence match first.
 
     :param filename: path to file
@@ -233,8 +251,9 @@ def magic_file(filename):
     return info
 
 
-def magic_string(string, filename=None):
-    """ Returns tuple of (num_of_matches, array_of_matches)
+def magic_string(string, filename: Union[os.PathLike, str] = None) -> List[PureMagicWithConfidence]:
+    """
+    Returns tuple of (num_of_matches, array_of_matches)
     arranged highest confidence match first
     If filename is provided it will be used in the computation.
 
@@ -251,8 +270,8 @@ def magic_string(string, filename=None):
     return info
 
 
-def magic_stream(stream, filename=None):
-    """ Returns tuple of (num_of_matches, array_of_matches)
+def magic_stream(stream, filename: Union[os.PathLike, str] = None) -> List[PureMagicWithConfidence]:
+    """Returns tuple of (num_of_matches, array_of_matches)
     arranged highest confidence match first
     If filename is provided it will be used in the computation.
 
